package cn.com.ty.lift.business.emergency.controller;


import cn.com.ty.lift.business.common.CountPage;
import cn.com.ty.lift.business.common.Values;
import cn.com.ty.lift.business.emergency.dto.RepairRequest;
import cn.com.ty.lift.business.emergency.dto.RepairResponse;
import cn.com.ty.lift.business.emergency.entity.EmergencyRepair;
import cn.com.ty.lift.business.emergency.service.impl.EmergencyRepairServiceImpl;
import cn.com.ty.lift.business.framework.conf.SystemConfiguration;
import cn.com.ty.lift.common.utils.AliyunOSSUtil;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;

/**
 * <p>
 * 临修记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/emergency/emergency-repair")
public class EmergencyRepairController {

    private EmergencyRepairServiceImpl emergencyRepairService;

    /**
     * @description 根据id查找
     * @date 2019/12/6 10:14
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody RepairRequest request){
        long id = request.getId();
        if(id == 0){
            return RestResponse.failParam();
        }
        RepairResponse result = emergencyRepairService.selectByIdWithInfo(request);
        if(null == result){
            return RestResponse.fail();
        }
        return RestResponse.success(result);
    }

    /**
     * @description 新增急修
     * @param
     * @return
     * @date 2019/12/9 9:43
     */
    @PostMapping("add")
    public RestResponse add(@RequestBody EmergencyRepair entity){

        if(null == entity){
            return RestResponse.failParam();
        }
        //派单时间 急修单创建时间
        entity.setAssignTime(LocalDateTime.now());
        boolean result = emergencyRepairService.save(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * @description 分页查询急修中的 status <> 4
     * @date 2019/12/9 9:39
     * @param 
     * @return 
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.listByRepairing(request);
        if(null == pages){
            return RestResponse.fail();
        }
        long count = pages.getTotal();
        CountPage<RepairResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 根据条件分页查询
     * @date 2019/12/9 9:39
     * @param 
     * @return 
     */
    @PostMapping("page")
    public RestResponse page(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.listByCondition(request);
        if(null == pages){
            return RestResponse.fail();
        }
        long count = emergencyRepairService.countByCondition(request);
        CountPage<RepairResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 急修单转派
     * @date 2019/12/11 9:25
     * @param
     * @return
     */
    @PostMapping("transfer")
    public RestResponse transfer(@RequestBody RepairRequest request){
        Long id = request.getId();
        Long workerId = request.getWorkerId();
        if(null == id || null == workerId){
            return RestResponse.failParam();
        }
        EmergencyRepair entity = emergencyRepairService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        //本急修单的负责维保工
        entity.setWorkerId1(workerId);
        boolean result = emergencyRepairService.saveOrUpdate(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * @description 关闭急修单
     * @date 2019/12/11 9:25
     * @param
     * @return
     */
    @PostMapping("close")
    public RestResponse close(@RequestBody RepairRequest request){
        Long id = request.getId();
        if(null == id){
            return RestResponse.failParam();
        }
        EmergencyRepair entity = emergencyRepairService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        //状态（0 待完成；1 已签到，待修理；2 已修理；3 待评价；4 完成 5 暂停中 6 关闭）
        entity.setStatus(Values.ER_STATUS_CLOSE);
        boolean result = emergencyRepairService.saveOrUpdate(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * app-其他角色获取自己关联的项目的急修任务
     * @param request
     * @return
     */
    @PostMapping("pageByUser")
    public RestResponse pageByUser(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByUser(request);
        if(null == pages){
            return RestResponse.fail();
        }
        return RestResponse.success(pages);
    }

    /**
     * app-点击接单，保存接单时间到急修表的接单时间接口
     * @param request
     * @return
     */
    @PostMapping("taking")
    public RestResponse taking(@RequestBody RepairRequest request){
        Long id = request.getId();
        LocalDateTime takingTime = request.getTakingTime();
        if(null == id || id < 0 || null == takingTime){
            return RestResponse.failParam();
        }
        EmergencyRepair entity = emergencyRepairService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setTakingTime(takingTime);
        boolean result = emergencyRepairService.updateById(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * app-点击到达，保存到达时间到急修表的到达时间中
     * @param request
     * @return
     */
    @PostMapping("arrive")
    public RestResponse arrive(@RequestBody RepairRequest request){
        Long id = request.getId();
        if(null == id || id < 0){
            return RestResponse.failParam();
        }
        LocalDateTime arriveTime = request.getArriveTime();
        if(null == arriveTime){
            return RestResponse.fail("缺少达到时间");
        }
        String position = request.getPosition();
        if(StrUtil.isEmpty(position)){
            return RestResponse.fail("缺少达到地点");
        }
        EmergencyRepair entity = emergencyRepairService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setArriveTime(arriveTime);
        entity.setStatus(Values.ER_STATUS_SIGN_IN);
        boolean result = emergencyRepairService.updateById(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * app-停梯时间+安全确认，保存停梯时间+安全确认项到急修表中
     * @param request
     * @return
     */
    @PostMapping("stop")
    public RestResponse stop(@RequestBody RepairRequest request){
        Long id = request.getId();
        LocalDateTime stopDate = request.getStopDate();
        if(null == id || id < 0 || null == stopDate){
            return RestResponse.failParam();
        }
        EmergencyRepair entity = emergencyRepairService.getById(id);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setStopDate(stopDate);
        boolean result = emergencyRepairService.updateById(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }
}
